@@ -5,7 +5,6 @@ __all__ = (
     "DuplicateResourceNames",
     "InvalidResource",
     "MissingResourceError",
-    "MissingTransactionBlock",
     "NestingUnitsOfWorkNotAllowed",
     "RollbackError",
 )
@@ -19,16 +18,6 @@ class LimeUoWException(Exception):
         super().__init__(message)
 
 
-class ClassMissingResourceNameOverride(LimeUoWException):
-    def __init__(self, class_name: str):
-        self.class_name = class_name
-        msg = (
-            f"The class, {class_name}, must override the __resource_name__ class attribute in "
-            f"order to use the .from_uow class method."
-        )
-        super().__init__(msg)
-
-
 class DuplicateResourceNames(LimeUoWException):
     def __init__(self, duplicates: typing.Mapping[str, int], /):
         self.duplicates = duplicates
@@ -59,11 +48,6 @@ class NestingUnitsOfWorkNotAllowed(LimeUoWException):
         )
 
 
-class MissingTransactionBlock(LimeUoWException):
-    def __init__(self, message: str):
-        super().__init__(message)
-
-
 class RollbackError(LimeUoWException):
     def __init__(self, message: str, /):
         super().__init__(message)
